from .cilindro import Cilindro
from .cono import Cono
from .cubo import Cubo
from .esfera import Esfera
from .flecha import Flecha
from .monopolo import Monopolo
from .paralelepipedo import Paralelepipedo
from .spines import Spines
from .tetrahedro import Tetrahedro